package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvElectricityDay;
import com.huimv.env.common.entity.EnvElectricityMonth;
import com.huimv.env.common.mapper.EnvElectricityDayMapper;
import com.huimv.env.common.mapper.EnvElectricityMonthMapper;
import com.huimv.env.common.service.EnvElectricityMonthService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.common.utils.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-21
 */
@Service
public class EnvElectricityMonthServiceImpl extends ServiceImpl<EnvElectricityMonthMapper, EnvElectricityMonth> implements EnvElectricityMonthService {
    @Resource
    private EnvElectricityMonthMapper envElectricityMonthMapper;
    @Resource
    private EnvElectricityDayMapper envElectricityDayMapper;

    @Override
    public EnvElectricityMonth getMonthElectricityConsumption(String farmCode, String deviceCode, String sensorSn, String year, String month) {
        QueryWrapper<EnvElectricityMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("year", year);
        queryWrapper.eq("month",month);
        return envElectricityMonthMapper.selectOne(queryWrapper);
    }

    @Override
    public void saveMonthElectricity(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, String year, String month, String farmCode) {
        String newElectricityValue = dataJo.getString("value");
        BigDecimal newElectricityBd = new BigDecimal(newElectricityValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        EnvElectricityMonth envElectricityMonth = new EnvElectricityMonth();
        envElectricityMonth.setDeviceCode(dataJo.getString("deviceCode"));
        envElectricityMonth.setStartTime(nowTimestamp);
        envElectricityMonth.setStartElectricity(newElectricityBd);
        envElectricityMonth.setEndTime(nowTimestamp);
        envElectricityMonth.setEndElectricity(newElectricityBd);
        envElectricityMonth.setMonthElectricity(new BigDecimal(0));
        envElectricityMonth.setDataUnit("立方米");
        envElectricityMonth.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envElectricityMonth.setUnitId(envDeviceRegisterEntity.getUnitId());
        envElectricityMonth.setYear(Integer.parseInt(year));
        envElectricityMonth.setMonth(Integer.parseInt(month));
        envElectricityMonth.setSensorSn(dataJo.getInteger("sensorSn"));
        envElectricityMonth.setLinkRelativeRatio(new BigDecimal(0));
        envElectricityMonth.setFarmCode(farmCode);
        int rows = envElectricityMonthMapper.insert(envElectricityMonth);
        System.out.println("添加记录数="+rows);
    }

    @Override
    public void updateMonthElectricity(float sumDayElectricityMeter, EnvElectricityMonth envElectricityMonth, JSONObject dataJo, Timestamp nowTimestamp) {
        BigDecimal endElectricityBd = new BigDecimal(dataJo.getString("value"));
        //BigDecimal newMonthElectricityBd = (endElectricityBd.subtract(envElectricityMonth.getStartElectricity())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        BigDecimal newMonthElectricityBd = new BigDecimal(sumDayElectricityMeter);
        BigDecimal linkRelativeRatioBd = new BigDecimal(0);
        try {
            linkRelativeRatioBd= newMonthElectricityBd.divide(envElectricityMonth.getStartElectricity(), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
        }catch (ArithmeticException e){
            System.out.println(linkRelativeRatioBd +"-->" + e);
        }
         envElectricityMonth.setEndTime(nowTimestamp);
        envElectricityMonth.setEndElectricity(endElectricityBd);
        envElectricityMonth.setMonthElectricity(newMonthElectricityBd);
        envElectricityMonth.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envElectricityMonthMapper.updateById(envElectricityMonth);
        System.out.println("编辑记录数="+rows);
    }

    @Override
    public Result getDayAndMonthElectricityList(Map<String, String> paramsMap) throws ParseException {
        String months = paramsMap.get("months");
        String days = paramsMap.get("days");
        if(StringUtils.isNotBlank(months) && StringUtils.isBlank(days)){
            //月份历史用电列表
            return new Result(ResultCode.SUCCESS,_getMonthElectricityList(paramsMap));
        }else if(StringUtils.isBlank(months) && StringUtils.isNotBlank(days)){
            //天数历史用电列表
            return new Result(ResultCode.SUCCESS,_getDayElectricityList(paramsMap));
        }else{
            return new Result(ResultCode.FAIL,"months和days参数异常，只能二选一。");
        }
    }

    //天数历史用电列表
    private Object _getDayElectricityList(Map<String, String> paramsMap) throws ParseException {
        String farmCode = paramsMap.get("farmCode");
        String days = paramsMap.get("days");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnvElectricityDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq(StringUtils.isNotBlank(unitId),"unit_id", unitId);
        queryWrapper.orderByDesc("add_date");
        queryWrapper.last("LIMIT "+days);
        List<EnvElectricityDay> envElectricityDayList = envElectricityDayMapper.selectList(queryWrapper);
        List monthList = new ArrayList();
        DateUtil dateUtil = new DateUtil();
        for(int a=envElectricityDayList.size()-1;a>=0;a--){
            EnvElectricityDay envElectricityDay = envElectricityDayList.get(a);
            Map<String,Object> map = new HashMap<>();
            map.put("datetime", dateUtil.formatDateText(envElectricityDay.getAddDate()));
            map.put("electricity",envElectricityDay.getDayElectricity());
            monthList.add(map);
        }
        return monthList;
    }

    //月份历史用电列表
    private Object _getMonthElectricityList(Map<String, String> paramsMap) {
        String farmCode = paramsMap.get("farmCode");
        String months = paramsMap.get("months");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnvElectricityMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq(StringUtils.isNotBlank(unitId),"unit_id", unitId);
        queryWrapper.orderByDesc("year").orderByDesc("month");
        queryWrapper.last("LIMIT "+months);
        List<EnvElectricityMonth> envElectricityMonthList = envElectricityMonthMapper.selectList(queryWrapper);
        List monthList = new ArrayList();
        for(int a=envElectricityMonthList.size()-1;a>=0;a--){
            EnvElectricityMonth envElectricityMonth = envElectricityMonthList.get(a);
            Map<String,Object> map = new HashMap<>();
            map.put("datetime",envElectricityMonth.getYear()+"-"+envElectricityMonth.getMonth());
            map.put("electricity",envElectricityMonth.getMonthElectricity());
            monthList.add(map);
        }
        return monthList;
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newWaterBd, String deviceCode, String sensorSn, String year, String month, String farmCode) {
        QueryWrapper<EnvElectricityMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.and(wrapper->wrapper.ne("year",year).or().ne("month",month));
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvElectricityMonth lastEnvElectricityMonth = envElectricityMonthMapper.selectOne(queryWrapper);
        if(lastEnvElectricityMonth == null){
            return new BigDecimal(0);
        }else{
            return (newWaterBd.subtract(lastEnvElectricityMonth.getMonthElectricity())).divide(lastEnvElectricityMonth.getMonthElectricity()).multiply(new BigDecimal(100));
        }
    }
}
