package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvWaterDay;
import com.huimv.env.common.entity.EnvWaterMonth;
import com.huimv.env.common.mapper.EnvWaterDayMapper;
import com.huimv.env.common.mapper.EnvWaterMonthMapper;
import com.huimv.env.common.service.EnvWaterDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.common.utils.Const;
import com.huimv.env.common.utils.DateUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-20
 */
@Service
public class EnvWaterDayServiceImpl extends ServiceImpl<EnvWaterDayMapper, EnvWaterDay> implements EnvWaterDayService {
    @Resource
    private EnvWaterDayMapper envWaterDayMapper;
    @Resource
    private EnvWaterMonthMapper envWaterMonthMapper;

    @Override
    public EnvWaterDay getDayWaterConsumption(String farmCode, String deviceCode, String sensorSn, Date todayDate) {
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("add_date", todayDate);
        return envWaterDayMapper.selectOne(queryWrapper);
    }

    @Override
    public Result getTodayWater(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        String deviceCode = paramsMap.get("deviceCode");
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
        queryWrapper.eq("add_date", dateUtil.getTodayDate());
        //读取当天的记录
        EnvWaterDay envWaterDay = envWaterDayMapper.selectOne(queryWrapper);
        if(envWaterDay == null){
            return new Result(Const.ERR_TODAY_WATER_CODE,Const.ERR_TODAY_WATER_INFO,false);
        }else{
            return new Result(ResultCode.SUCCESS,envWaterDay);
        }
    }

    @Override
    public Result getWaterCount(Map<String, String> paramsMap) throws ParseException {
        //-- 今日用水统计(今日用水消耗和环比) --//
        EnvWaterDay envWaterDay = _getTodayWaterCount(paramsMap);
        BigDecimal todayCount = new BigDecimal(0);
        BigDecimal todayLinkRelativeRatio = new BigDecimal(0);
        if(envWaterDay != null){
            todayCount = envWaterDay.getDayWater();
            todayLinkRelativeRatio = envWaterDay.getLinkRelativeRatio();
        }
        //-- 本月用水统计(本月用水消耗和环比) --//
        EnvWaterMonth envWaterMonth = _getMonthWaterCount(paramsMap);
        BigDecimal thisMonthCount = new BigDecimal(0);
        BigDecimal thisMonthLinkRelativeRatio = new BigDecimal(0);
        if(envWaterMonth != null){
            thisMonthCount = envWaterMonth.getMonthWater();
            thisMonthLinkRelativeRatio = envWaterMonth.getLinkRelativeRatio();
        }
        JSONObject resultJo = new JSONObject();
        resultJo.put("todayCount",todayCount);
        resultJo.put("todayLinkRelativeRatio",todayLinkRelativeRatio);
        resultJo.put("thisMonthCount",thisMonthCount);
        resultJo.put("thisMonthLinkRelativeRatio",thisMonthLinkRelativeRatio);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

    @Override
    public float getSumWaterMeter(String farmCode, String deviceCode, Date startDate, Date endDate) {
        DateUtil dateUtil = new DateUtil();
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.select("IFNULL(sum(day_water),0) as total");
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code",deviceCode);
        queryWrapper.between("add_date",startDate,endDate);
        Map<String, Object> map = this.getMap(queryWrapper);
        BigDecimal sumBd = (BigDecimal) map.get("total");
        return sumBd.floatValue();
    }

    //获取本月用水统计
    private EnvWaterMonth _getMonthWaterCount(Map<String, String> paramsMap) {
        String farmCode = paramsMap.get("farmCode");
        DateUtil dateUtil = new DateUtil();
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("year",dateUtil.getThisYear());
        queryWrapper.eq("month",dateUtil.getThisMonth());
//        queryWrapper.orderByDesc("year").orderByDesc("month");
//        queryWrapper.last("LIMIT 1");
        return envWaterMonthMapper.selectOne(queryWrapper);
    }

    //获取今天用水统计
    private EnvWaterDay _getTodayWaterCount(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("add_date", dateUtil.getTodayDate());
        //读取当天的记录(每天本牧场所有用水统计)
        EnvWaterDay envWaterDay = envWaterDayMapper.selectOne(queryWrapper);
        return envWaterDay;
    }

    @Override
    public void saveDayWater(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        String newWaterValue = dataJo.getString("value");
        BigDecimal newWaterBd = new BigDecimal(newWaterValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取环比
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newWaterBd, deviceCode,  sensorSn,  todayDate,  farmCode);
        EnvWaterDay envWaterDay = new EnvWaterDay();
        envWaterDay.setDeviceCode(dataJo.getString("deviceCode"));
        envWaterDay.setStartTime(nowTimestamp);
        envWaterDay.setStartWater(newWaterBd);
        envWaterDay.setEndTime(nowTimestamp);
        envWaterDay.setEndWater(newWaterBd);
        envWaterDay.setDayWater(new BigDecimal(0));
        envWaterDay.setDataUnit("立方米");
        envWaterDay.setSensorSn(dataJo.getInteger("sensorSn"));
        envWaterDay.setAddDate(todayDate);
        envWaterDay.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envWaterDay.setUnitId(envDeviceRegisterEntity.getUnitId());
        envWaterDay.setLinkRelativeRatio(linkRelativeRatioBd);
        envWaterDay.setFarmCode(farmCode);
        System.out.println("保存每天水量 envWaterDay >> "+envWaterDay);
        int rows = envWaterDayMapper.insert(envWaterDay);
        System.out.println("添加记录数="+rows);
    }

    @Override
    public void updateDayWater(float sumWaterMeter, EnvWaterDay envWaterDay, JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        BigDecimal endWaterBd = new BigDecimal(dataJo.getString("value"));
//        BigDecimal newDayWaterBd = (endWaterBd.subtract(envWaterDay.getStartWater())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //计算累加结果
        BigDecimal newDayWaterBd = new BigDecimal(sumWaterMeter);
        //获取环比
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newDayWaterBd, deviceCode,  sensorSn,  todayDate,  farmCode);
        envWaterDay.setEndTime(nowTimestamp);
        envWaterDay.setEndWater(endWaterBd);
        envWaterDay.setDayWater(newDayWaterBd);
        envWaterDay.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envWaterDayMapper.updateById(envWaterDay);
        System.out.println("编辑记录数="+rows);
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newWaterBd, String deviceCode, String sensorSn, Date todayDate, String farmCode) {
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.ne("add_date", todayDate);
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvWaterDay lastEnvWaterDay = envWaterDayMapper.selectOne(queryWrapper);
        if(lastEnvWaterDay == null){
            return new BigDecimal(0);
        }else{
            if(lastEnvWaterDay.getDayWater().toString().equalsIgnoreCase("0.00")){
                return new BigDecimal(0);
            }else{
                return (newWaterBd.subtract(lastEnvWaterDay.getDayWater())).divide(lastEnvWaterDay.getDayWater(),2,BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
            }
        }
    }
}
