package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvWaterDay;
import com.huimv.env.common.entity.EnvWaterMonth;
import com.huimv.env.common.mapper.EnvWaterDayMapper;
import com.huimv.env.common.mapper.EnvWaterMonthMapper;
import com.huimv.env.common.service.EnvWaterMonthService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.common.utils.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-20
 */
@Service
public class EnvWaterMonthServiceImpl extends ServiceImpl<EnvWaterMonthMapper, EnvWaterMonth> implements EnvWaterMonthService {
    @Resource
    private EnvWaterMonthMapper envWaterMonthMapper;
    @Resource
    private EnvWaterDayMapper envWaterDayMapper;

    @Override
    public EnvWaterMonth getMonthWaterConsumption(String farmCode, String deviceCode, String sensorSn, String year, String month) {
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("year", year);
        queryWrapper.eq("month",month);
        return envWaterMonthMapper.selectOne(queryWrapper);
    }

    @Override
    public Result getDayAndMonthWaterList(Map<String, String> paramsMap) throws ParseException {
        String farmCode = paramsMap.get("farmCode");
        String months = paramsMap.get("months");
        String unitId = paramsMap.get("unitId");
        String days = paramsMap.get("days");
        if(StringUtils.isNotBlank(months) && StringUtils.isBlank(days)){
            //查看12个月用水记录
            return new Result(ResultCode.SUCCESS,_getMonthWaterList(farmCode,months,unitId));
        }else if(StringUtils.isBlank(months) && StringUtils.isNotBlank(days)){
            //查看30天用水记录
            return new Result(ResultCode.SUCCESS,_getDayWaterList(farmCode,days,unitId));
        }else{
            System.out.println("months="+months);
            System.out.println("days="+days);
            return new Result(ResultCode.FAIL,"months和days两个参数只能二选一。");
        }
    }

    //查看30天用水记录
    private Object _getDayWaterList(String farmCode, String days,String unitId) throws ParseException {
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.orderByDesc("add_date");
        queryWrapper.eq(StringUtils.isNotBlank(unitId),"unit_id", unitId);
        queryWrapper.last("LIMIT "+days);
        List<EnvWaterDay> envWaterDayList = envWaterDayMapper.selectList(queryWrapper);
        List dayList = new ArrayList();
        DateUtil dateUtil = new DateUtil();
        for(int a=envWaterDayList.size()-1;a>=0;a--){
            EnvWaterDay envWaterDay = envWaterDayList.get(a);
            Map<String,Object> waterMap = new HashMap();
            waterMap.put("datetime",dateUtil.formatDateText(envWaterDay.getAddDate()));
            waterMap.put("water",envWaterDay.getDayWater());
            dayList.add(waterMap);
        }
        return dayList;
    }

    //查看12个月用水记录
    private Object _getMonthWaterList(String farmCode, String months,String unitId) {
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq(StringUtils.isNotBlank(unitId),"unit_id", unitId);
        queryWrapper.orderByDesc("year").orderByDesc("month");
        queryWrapper.last("LIMIT "+months);
//        List<Map<String,Object>> envWaterMonthList = envWaterMonthMapper.getWaterList();
        List<EnvWaterMonth> envWaterMonthList = envWaterMonthMapper.selectList(queryWrapper);
        List monthList = new ArrayList();
        for(int a=envWaterMonthList.size()-1;a>=0;a--){
            EnvWaterMonth envWaterMonth = envWaterMonthList.get(a);
            Map<String,Object> waterMonthMap = new HashMap<>();
            waterMonthMap.put("datetime",envWaterMonth.getYear()+"-"+envWaterMonth.getMonth());
            waterMonthMap.put("water",envWaterMonth.getMonthWater());
            monthList.add(waterMonthMap);
        }
        return monthList;
    }

    @Override
    public void saveMonthWater(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, String year, String month, String farmCode) {
        String newWaterValue = dataJo.getString("value");
        BigDecimal newWaterBd = new BigDecimal(newWaterValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取月份水量环比
        BigDecimal linkRelativeRatio = _getLinkRelativeRatio(newWaterBd,deviceCode,sensorSn,year,month,farmCode);
        EnvWaterMonth envWaterMonth = new EnvWaterMonth();
        envWaterMonth.setDeviceCode(deviceCode);
        envWaterMonth.setStartTime(nowTimestamp);
        envWaterMonth.setStartWater(newWaterBd);
        envWaterMonth.setEndTime(nowTimestamp);
        envWaterMonth.setEndWater(newWaterBd);
        envWaterMonth.setMonthWater(new BigDecimal(0));
        envWaterMonth.setDataUnit("立方米");
        envWaterMonth.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envWaterMonth.setUnitId(envDeviceRegisterEntity.getUnitId());
        envWaterMonth.setYear(Integer.parseInt(year));
        envWaterMonth.setMonth(Integer.parseInt(month));
        envWaterMonth.setSensorSn(dataJo.getInteger("sensorSn"));
        envWaterMonth.setLinkRelativeRatio(linkRelativeRatio);
        envWaterMonth.setFarmCode(farmCode);
        int rows = envWaterMonthMapper.insert(envWaterMonth);
        System.out.println("添加记录数="+rows);
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newWaterBd, String deviceCode, String sensorSn, String year, String month, String farmCode) {
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.and(wrapper->wrapper.ne("year",year).or().ne("month",month));
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvWaterMonth lastEnvWaterMonth = envWaterMonthMapper.selectOne(queryWrapper);
        if(lastEnvWaterMonth == null){
            return new BigDecimal(0);
        }else{
            if(lastEnvWaterMonth.getMonthWater().toString().equalsIgnoreCase("0.00")){
                return new BigDecimal(0);
            }else{
                return (newWaterBd.subtract(lastEnvWaterMonth.getMonthWater())).divide(lastEnvWaterMonth.getMonthWater(), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
            }
        }
    }

    @Override
    public void updateMonthWater(float sumDayWaterMeter, EnvWaterMonth envWaterMonth, JSONObject dataJo, Timestamp nowTimestamp, String year, String month, String farmCode) {
        BigDecimal endWaterBd = new BigDecimal(dataJo.getString("value"));
//        BigDecimal newMonthWaterBd = (endWaterBd.subtract(envWaterMonth.getStartWater())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        BigDecimal newMonthWaterBd = new BigDecimal(sumDayWaterMeter);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newMonthWaterBd,deviceCode,sensorSn,year,month,farmCode);
        envWaterMonth.setEndTime(nowTimestamp);
        envWaterMonth.setEndWater(endWaterBd);
        envWaterMonth.setMonthWater(newMonthWaterBd);
        envWaterMonth.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envWaterMonthMapper.updateById(envWaterMonth);
        System.out.println("编辑记录数="+rows);
    }
}
