package com.huimv.env.device.listener;

import com.alibaba.fastjson.JSONObject;
import com.huimv.env.common.dao.entity.EnvAmmoniaEntity;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.dao.entity.EnvHumiEntity;
import com.huimv.env.common.dao.entity.EnvTempEntity;
import com.huimv.env.common.dao.repo.EnvAmmoniaEntityRepo;
import com.huimv.env.common.dao.repo.EnvHumiEntityRepo;
import com.huimv.env.common.dao.repo.EnvTempEntityRepo;
import com.huimv.env.common.entity.BaseThreshold;
import com.huimv.env.common.service.*;
import com.huimv.env.common.utils.Const;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.common.utils.MathUtil;
import com.huimv.env.common.utils.MathUtil2;
import com.huimv.env.device.config.WeChatMessageHelperTest;
import com.huimv.env.device.config.WebSocket;
import com.huimv.env.device.producer.Producer;
import com.huimv.env.device.service.ICommandParserService;
import com.huimv.env.device.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class DeviceListener {
    @Autowired
    private IDeviceRegisterService deviceRegisterService;
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private MathUtil2 mathUtil;
    @Autowired
    private ICommandParserService cmdParserService;
    @Autowired
    private IPushMessageService pushMessageService;
    @Autowired
    private ISprayStatusService sprayStatusService;
    @Autowired
    private IEnvPushMessageService envPushMessageService;


    @Autowired
    private WebSocket webSocket;

    @RabbitListener(queues = Const.QUEUE_ELECTRICITY_METER)
    @RabbitHandler
    public void processElectricityMeter(Map RawMap)   {
        System.out.println(">>>>>>>>>>>>>> 222222222222222222 接收电表数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "");
        System.out.println("电表数据=" + dataJo);
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String electricityMeter =  new BigDecimal(mathUtil.countMeter(dataJo.getIntValue("value"))).setScale(1, RoundingMode.UP)+dataJo.getString("dataUnit");;
        int sensorSort = 1;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, electricityMeter, farmCode,sensorSort);

        //保存实时电表流水数据
        deviceService.saveElectricityMeter(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);

        //保存每天用电量数据
        deviceService.saveDayElectricityConsumption(dataJo,envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);

        //保存每月用电量数据
        deviceService.saveMonthElectricityConsumption(dataJo,envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
    }


    @RabbitListener(queues = Const.QUEUE_WATER_METER)
    @RabbitHandler
    public void processWaterMeterGege(Map RawMap) throws ParseException {
        System.out.println(" 接收水表数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "");
        System.out.println("水表数据=" + dataJo);
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        DateUtil dateUtil = new DateUtil();
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String waterMeter =  new BigDecimal(mathUtil.countMeter(dataJo.getIntValue("value"))).setScale(1, RoundingMode.UP)+dataJo.getString("dataUnit");;
        int sensorSort = 1;
         //
        //保存水表流水数据
        deviceService.saveWaterMeter(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);

        //保存每天用水量数据
        deviceService.saveDayWaterConsumption(dataJo,envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);

        //保存每月用水量数据
        deviceService.saveMonthWaterConsumption(dataJo,envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);

        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, waterMeter, farmCode,sensorSort);

    }
    
    @RabbitListener(queues = Const.QUEUE_PH)
    @RabbitHandler
    public void processPhGege(Map RawMap)   {
        System.out.println(">>>>>>>>>>>>>>接收PH数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "");
        System.out.println("PH数据=" + dataJo);
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String PH = new BigDecimal(mathUtil.countMeter(dataJo.getIntValue("value"))).setScale(1, RoundingMode.UP)+dataJo.getString("dataUnit");

        int sensorSort = 1;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, PH, farmCode,sensorSort);

        //保存PH流水数据
        deviceService.savePH(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
    }


    @RabbitListener(queues = Const.QUEUE_WATER_GEGE)
    @RabbitHandler
    public void processWaterGege(Map RawMap)   {
        System.out.println(">>>>>>>>>>>>>>接收水压数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "Kpa");
        System.out.println("水压数据=" + dataJo);
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
       String watreGege = dataJo.getString("value");
        int sensorSort = 1;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, watreGege, farmCode,sensorSort);

        //保存水压流水数据
        deviceService.saveWaterGege(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
    }


    @RabbitListener(queues = Const.QUEUE_TEMP)
    @RabbitHandler
    public void processTemp(Map RawMap) throws ParseException {
        System.out.println(">>>>>>>>>>>>>>接收温度数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "°");
        System.out.println("温度数据=" + dataJo);
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String temp = new BigDecimal(mathUtil.countTemp(dataJo.getIntValue("value"))).setScale(1, RoundingMode.UP) + dataJo.getString("dataUnit");
        int sensorSort = 1;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, temp, farmCode,sensorSort);
        //报警 TODO
//        envPushMessageService.saveTemp();
        //保存温度流水数据
        deviceService.saveTempFlow(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
    }

    @RabbitListener(queues = Const.QUEUE_HUMI)
    @RabbitHandler
    public void processHumi(Map RawMap) throws ParseException {
        System.out.println(">>>>>>>>>>>>>>接收湿度数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "%");
        System.out.println("湿度数据=" + dataJo);
        String todayDateText = new DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //根据设备编码获取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if (envDeviceRegisterEntity == null) {
            log.error("该设备[" + dataJo.getString("deviceCode") + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String humi = new BigDecimal(mathUtil.countTemp(dataJo.getIntValue("value"))).setScale(1, RoundingMode.UP) + dataJo.getString("dataUnit");
        int sensorSort = 2;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, humi, farmCode, sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, humi, farmCode, sensorSort);
        //保存湿度流水数据
        deviceService.saveHumiFlow(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
        envPushMessageService.saveHumi(deviceCode,nowTimestamp,dataJo,farmCode);
    }

    @RabbitListener(queues = Const.QUEUE_AMMONIA)
    @RabbitHandler
    public void processAmmonia(Map RawMap) throws ParseException {
        System.out.println(">>>>>>>>>>>>>>接收氨气数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleAskText(askText);
        dataJo.put("dataUnit", "ppm");
        System.out.println("氨气数据=" + dataJo);
        String todayDateText = new DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String deviceCode = dataJo.getString("deviceCode");
        //读取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if(envDeviceRegisterEntity == null){
            log.error("该设备[" + deviceCode + "]未注册.");
            return;
        }
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        String anmonia = dataJo.getString("value") + dataJo.getString("dataUnit");
        int sensorSort = 3;
        //更新传感器数据
//        deviceService.updateSensorRegister(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, anmonia, farmCode, sensorSort);
        deviceService.sendSensorToMQ(deviceCode, dataJo.getInteger("sensorSn"), nowTimestamp, todayDate, anmonia, farmCode, sensorSort);
        //保存氨气流水数据
        deviceService.saveAmmoniaFlow(dataJo, envDeviceRegisterEntity, nowTimestamp, todayDate, farmCode);
    }

    @RabbitListener(queues = Const.QUEUE_DEVICE_STATUS)
    @RabbitHandler
    public void processSprayStatus(Map RawMap) throws ParseException {
        System.out.println(">>>>>>>>>>>>>>接收设备状态数据 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = deviceService.handleSprayStatusAskCmd(askText);
        String deviceCode = dataJo.getString("deviceCode");
        //读取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if(envDeviceRegisterEntity == null){
            log.error("该设备[" + deviceCode + "]未注册.");
            return;
        }
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        //{保存喷雾状态流水}
        sprayStatusService.saveSprayStatusFlow(dataJo,envDeviceRegisterEntity,nowTimestamp,todayDate,farmCode);

    }


    @RabbitListener(queues = Const.QUEUE_PUSH_MESSAGE)
    @RabbitHandler
    public void processPushMessage(Map RawMap) throws ParseException {
        System.out.println(">>>>>>>>>>>>>>接收平台推送消息 RawMap>>" + RawMap.toString());
        String askText = RawMap.get("askText").toString();
        JSONObject dataJo = cmdParserService.handlePushMessageCmd(askText);
        String deviceCode = dataJo.getString("deviceCode");
        //读取设备注册信息
        EnvDeviceRegisterEntity envDeviceRegisterEntity = deviceRegisterService.getDeviceRegisterByDeviceCode(deviceCode);
        if(envDeviceRegisterEntity == null){
            log.error("该设备[" + deviceCode + "]未注册.");
            return;
        }
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        String farmCode = envDeviceRegisterEntity.getFarmCode();
        //webSocket 推送
        webSocket.sendToUser("true,"+farmCode);
        //微信推送
//        WeChatMessageHelperTest.sendWechatAppSubMessage();


        //{}
        String messageContent = deviceService.getMessageContent(dataJo,envDeviceRegisterEntity);
        //{保存消息推送流水}
        pushMessageService.savePushMessageFlow(dataJo,messageContent,nowTimestamp,todayDate,farmCode);
    }




}
