package com.huimv.env.manage.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.env.manage.entity.EnvDeviceRegister;
import com.huimv.env.manage.entity.dto.EnvDeviceRegisterDto;
import com.huimv.env.manage.mapper.BasePigpenMapper;
import com.huimv.env.manage.mapper.EnvDeviceRegisterMapper;
import com.huimv.env.manage.mapper.LampConfigMapper;
import com.huimv.env.manage.saas.dao.entity.BasePigpen;
import com.huimv.env.manage.mapper.EnvDeviceEquipmentMapper;
import com.huimv.env.manage.saas.dao.entity.EnvDeviceEquipment;
import com.huimv.env.manage.saas.dao.entity.LampConfig;
import com.huimv.env.manage.service.IEnvDeviceRegisterService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.manage.utils.Result;
import com.huimv.env.manage.utils.ResultCode;
import com.huimv.env.manage.utils.ResultUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-08-18
 */
@Service
public class EnvDeviceRegisterServiceImpl extends ServiceImpl<EnvDeviceRegisterMapper, EnvDeviceRegister> implements IEnvDeviceRegisterService {

    @Autowired
    private EnvDeviceRegisterMapper envDeviceRegisterMapper;

    @Autowired
    private EnvDeviceEquipmentMapper envDeviceEquipmentMapper;

    @Autowired
    private LampConfigMapper lampConfigMapper;

    @Autowired
    private BasePigpenMapper basePigpenMapper;
    @Override
    public Result registerList(Map<String, String> paramMap) {
        String farmCode = paramMap.get("farmCode");
        String pageSize = paramMap.get("pageSize");
        String pageNum = paramMap.get("pageNum");
        if (StringUtils.isBlank(pageNum)){
            pageNum = "1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize = "20";
        }
        IPage<EnvDeviceRegisterDto> envDeviceRegisterPage = new Page<>(Long.parseLong(pageNum), Long.parseLong(pageSize));
       IPage<EnvDeviceRegisterDto> page=  envDeviceRegisterMapper.registerList(envDeviceRegisterPage,farmCode);
        for (EnvDeviceRegisterDto record : page.getRecords()) {
            if (ObjectUtil.isEmpty(record.getHumi())){
                record.setHumi(new BigDecimal(0));
            }
            if (ObjectUtil.isEmpty(record.getTemp())){
                record.setTemp(new BigDecimal(0));
            }
            if (ObjectUtil.isEmpty(record.getAmmonia())){
                record.setAmmonia(new BigDecimal(0));
            }
        }
        return new Result(ResultCode.SUCCESS, page);
    }

    @Override
    @Transactional
    public Result saveRegister(Map<String, String> paramMap) {
        String deviceCode = paramMap.get("deviceCode");
        deviceCode = deviceCode.substring(2);
        String unitId = paramMap.get("unitId");
        String farmCode = paramMap.get("farmCode");
        String deviceList = paramMap.get("deviceList");

       if (envDeviceRegisterMapper.exists(new QueryWrapper<EnvDeviceRegister>().eq("chip_id",deviceCode))){
           return new Result(10001,"该设备已存在",false);
       }
        if (envDeviceRegisterMapper.exists(new QueryWrapper<EnvDeviceRegister>().eq("unit_id",unitId))){
            return new Result(10001,"该单元已绑定设备",false);
        }

        EnvDeviceRegister deviceRegister1 = envDeviceRegisterMapper.selectOne(new QueryWrapper<EnvDeviceRegister>().orderByDesc("device_code").last("limit 1"));
        String deviceCode1;
        if (ObjectUtil.isEmpty(deviceRegister1)){
            deviceCode1 ="1";
        }else {
            deviceCode1 = (Integer.parseInt(deviceRegister1.getDeviceCode())+1) +"";
        }

        EnvDeviceRegister deviceRegister = new EnvDeviceRegister();
        deviceRegister.setChipId(deviceCode);
        deviceRegister.setUnitId(unitId);
        deviceRegister.setFarmCode(farmCode);
        deviceRegister.setAddTime(new Date());
        deviceRegister.setDeviceCode(deviceCode1);

        envDeviceRegisterMapper.insert(deviceRegister);

        String[] split = deviceList.split(",");
        int port = 1;
        for (String s : split) {
            EnvDeviceEquipment envDeviceEquipment = new EnvDeviceEquipment();
            envDeviceEquipment.setEquipmentName(s);
            envDeviceEquipment.setUnitId(Integer.parseInt(unitId));
            envDeviceEquipment.setDeviceCode(deviceCode1);
            envDeviceEquipment.setEquipmentPort(port);
//            envDeviceEquipment.setDeviceAlias(port);
            envDeviceEquipmentMapper.insert(envDeviceEquipment);
            port++;
        }

        return new Result(ResultCode.SUCCESS,true);
    }

    @Override
    public Result listByUnit(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        List<EnvDeviceEquipment> list = envDeviceEquipmentMapper.selectList(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId));

        return ResultUtil.isNull(list);
    }

    @Override
    @Transactional
    public Result unbindingBaseDeviceCofig(Map<String, String> map, HttpServletRequest request) {
        String deviceCode = map.get("deviceCode");
        EnvDeviceRegister deviceRegister = new EnvDeviceRegister();
        deviceRegister.setUnitId("0");
        deviceRegister.setBinding(0);
        deviceRegister.setPigpenId("0");
        envDeviceRegisterMapper.update(deviceRegister,new UpdateWrapper<EnvDeviceRegister>().eq("device_code",deviceCode));

        EnvDeviceEquipment envDeviceEquipment = new EnvDeviceEquipment();
        envDeviceEquipment.setUnitId(0);
        envDeviceEquipment.setUploadStart(0);
        envDeviceEquipment.setBinding(0);
        envDeviceEquipmentMapper.update(envDeviceEquipment,new UpdateWrapper<EnvDeviceEquipment>().eq("device_code",deviceCode));
        return new Result(10000,"解绑成功",true);
    }

    @Override
    public Result lsitAllDeivce(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");
        List<EnvDeviceRegister> envDeviceRegisterList = envDeviceRegisterMapper.selectList(new QueryWrapper<EnvDeviceRegister>().eq("farm_code", farmCode));
        List endList = new ArrayList();
        for (EnvDeviceRegister deviceRegister : envDeviceRegisterList) {
            Map endMap = new HashMap();
            String deviceCode = deviceRegister.getDeviceCode();
            endMap.put("chipId",deviceRegister.getChipId());
            endMap.put("id",deviceRegister.getId());
            endMap.put("deviceCode",deviceCode);
            BasePigpen basePigpen = basePigpenMapper.selectById(deviceRegister.getUnitId());
            if (ObjectUtil.isNotEmpty(basePigpen)){
                endMap.put("unitName",basePigpen.getPigpenName());
            }else {
                endMap.put("unitName","");
            }

            endMap.put("unitId",deviceRegister.getUnitId());
            endMap.put("equipmentList",envDeviceEquipmentMapper.selectList(new QueryWrapper<EnvDeviceEquipment>().eq("device_code",deviceCode)));
            endMap.put("binding",deviceRegister.getBinding());

            endList.add(endMap);
        }


        return ResultUtil.isNull(endList);
    }

    @Override
    @Transactional
    public Result bindingBaseDeviceCofig(Map<String, String> map, HttpServletRequest request) {
        String deviceCode = map.get("deviceCode");
        String unitId = map.get("unitId");
        if (envDeviceRegisterMapper.exists(new QueryWrapper<EnvDeviceRegister>().eq("unit_id",unitId))){
            return new Result(10001,"该单元已绑定设备",false);
        }
        EnvDeviceRegister deviceRegister = new EnvDeviceRegister();
        deviceRegister.setUnitId(unitId);
        deviceRegister.setBinding(1);
        envDeviceRegisterMapper.update(deviceRegister,new UpdateWrapper<EnvDeviceRegister>().eq("device_code",deviceCode));

        EnvDeviceEquipment envDeviceEquipment = new EnvDeviceEquipment();
        envDeviceEquipment.setUnitId(Integer.parseInt(unitId));
        envDeviceEquipment.setUploadStart(1);
        envDeviceEquipment.setBinding(1);
        envDeviceEquipmentMapper.update(envDeviceEquipment,new UpdateWrapper<EnvDeviceEquipment>().eq("device_code",deviceCode));
        return new Result(10000,"绑定成功",true);
    }

    @Override
    public Result updateAllByUnitId(Map<String, Integer> paramMap) {
        Integer unitId = paramMap.get("unitId");
        Integer uploadStart = paramMap.get("uploadStart");
        EnvDeviceEquipment envDeviceEquipment = new EnvDeviceEquipment();
        envDeviceEquipment.setUploadStart(uploadStart);
        envDeviceEquipmentMapper.update(envDeviceEquipment,new QueryWrapper<EnvDeviceEquipment>().eq("unit_id",unitId));
        return new Result(10000,"修改成功",true);
    }


    @Override
    public Result screenDeviceCofig(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        List list = new ArrayList();
        boolean exists1 = lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id", unitId).eq("upload_start", 1));
        boolean exists2 = lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id", unitId).ne("board_gear", 0));
        boolean exists3 = envDeviceEquipmentMapper.exists(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId).eq("upload_start", 1).eq("equipment_name", "照明"));
        boolean exists4 = envDeviceEquipmentMapper.exists(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId).eq("upload_start", 1).eq("equipment_name", "风机"));
        boolean exists5 = envDeviceEquipmentMapper.exists(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId).eq("upload_start", 1).eq("equipment_name", "刮粪板"));
        list.add(exists1);
        list.add(exists2);
        list.add(exists3);
        list.add(exists4);
        list.add(exists5);
        return new Result(ResultCode.SUCCESS,list);
    }
}
