package com.huimv.env.produce.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.produce.service.DataSourceService;
import com.huimv.env.produce.service.StockDetailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zn
 * @since 2022-11-01
 */
@RestController
@RequestMapping("/v1.0.0/stock")
public class StockDetailController {
    @Autowired
    private StockDetailService stockDetailService;
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private RestTemplate restTemplate;


    //查询
    @PostMapping("/getStockHistory")
    public Result getStockHistory(@RequestBody Map<String, String> paramsMap) throws ParseException {
        Integer dsStatu = dataSourceService.getDataSourceStatus(paramsMap);
        if (dsStatu == null) {
            dsStatu = 0;
        }
        if (dsStatu == 1) {
            //读取远程历史存栏
            return getRemoteHistoryStock(paramsMap);
        } else {
            //读取填报数据
            return stockDetailService.getLocalHistoryStock(paramsMap);
        }
    }

    //
    private Result getRemoteHistoryStock(Map<String, String> paramsMap) {
        HttpHeaders httpHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json;charset=UTF-8");
        httpHeaders.setContentType(type);
//        MultiValueMap<String, Object> map=new LinkedMultiValueMap<>();
        HashMap<String, Object> map = new HashMap<>();
        map.put("farmCode", paramsMap.get("farmCode"));
        map.put("months", paramsMap.get("months"));
        HttpEntity<Map<String, Object>> objectHttpEntity = new HttpEntity<>(map, httpHeaders);
        //读取真实数据
        String remoteUrl = "http://123.60.134.84:10001/breed/api/v1.0.0/stock/getStockHistory";
        ResponseEntity<String> entity = restTemplate.postForEntity(remoteUrl, objectHttpEntity, String.class);
        if(entity.getStatusCode().toString().contains("200")){
            String body = entity.getBody();
            return new Result(ResultCode.SUCCESS,JSONArray.parseArray(JSON.parseObject(body).getString("data")));
        }else{
            return new Result(ResultCode.FAIL,"调取远程数据出错.");
        }
    }

    //查询
    @PostMapping("/getCurrentStock")
    public Result getCurrentStock(@RequestBody Map<String, String> paramsMap) throws ParseException {
        Integer dsStatu = dataSourceService.getDataSourceStatus(paramsMap);
        if (dsStatu == null) {
            dsStatu = 0;
        }
        //dsStatu=1,dsStatu=2
        if (dsStatu == 1) {
            //dsStatu=1 ,真实数据
            System.out.println("大屏-存栏结构-读取远程数据");
            //读取远程数据
            return getRemoteStock(paramsMap);
        } else {
            System.out.println("大屏-存栏结构-读取本地填报数据");
            //dsStatu=0 ,填报数据
            //读取填报数据
            return stockDetailService.getCurrentStock(paramsMap);
        }
    }

    //读取远程数据
    public Result getRemoteStock(Map<String, String> paramsMap){
        HttpHeaders httpHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json;charset=UTF-8");
        httpHeaders.setContentType(type);
//        MultiValueMap<String, Object> map=new LinkedMultiValueMap<>();
        HashMap<String, Object> map = new HashMap<>();
        map.put("farmCode", paramsMap.get("farmCode"));
        HttpEntity<Map<String, Object>> objectHttpEntity = new HttpEntity<>(map, httpHeaders);
        //读取真实数据
        String remoteUrl = "http://123.60.134.84:10001/breed/api/v1.0.0/stock/getCurrentStock";
        ResponseEntity<String> entity = restTemplate.postForEntity(remoteUrl, objectHttpEntity, String.class);
        if(entity.getStatusCode().toString().contains("200")){
            String body = entity.getBody();
            return new Result(ResultCode.SUCCESS,JSONArray.parseArray(JSON.parseObject(body).getString("data")));
        }else{
            return new Result(ResultCode.FAIL,"调取远程数据出错.");
        }
    }

    //查询
    @PostMapping("/getStock")
    public Result getStock(@RequestBody Map<String, String> paramsMap) throws ParseException {
        return stockDetailService.getStock(paramsMap);
    }

    //新增
    @PostMapping("/save")
    public Result save(@RequestBody Map<String, String> paramsMap) throws ParseException {
        return stockDetailService.save(paramsMap);
    }

    //删除
    @PostMapping("/remove")
    public Result remove(@RequestBody Map<String, String> paramsMap) throws ParseException {
        return stockDetailService.remove(paramsMap);
    }
}

