package com.huimv.env.produce.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.produce.entity.Eliminate;
import com.huimv.env.produce.entity.EliminateDetail;
import com.huimv.env.produce.mapper.EliminateDetailMapper;
import com.huimv.env.produce.mapper.EliminateMapper;
import com.huimv.env.produce.service.EliminateDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-11-01
 */
@Service
public class EliminateDetailServiceImpl extends ServiceImpl<EliminateDetailMapper, EliminateDetail> implements EliminateDetailService {
    @Autowired
    private EliminateMapper eliminateMapper;
    @Autowired
    private EliminateDetailMapper eliminateDetailMapper;
    //默认牧场
    private static final String DEFAULT_FARM_CODE = "330211";

    @Override
    public Result getEliminateCount(Map<String, String> paramsMap) {
        //
        String farmCode = paramsMap.get("farmCode");
        if(farmCode == null){
            farmCode = DEFAULT_FARM_CODE;
        }
        JSONObject resultJo = new JSONObject();
        //查询淘汰总数
        QueryWrapper<Eliminate> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code",farmCode);
        Eliminate eliminate = eliminateMapper.selectOne(queryWrapper);
        if(eliminate == null){
            resultJo.put("all",0);
            resultJo.put("boarCount",0);
            resultJo.put("sowCount",0);
            resultJo.put("fatpigsCount",0);
        }else{
            resultJo.put("all",eliminate.getTotalCount());
            resultJo.put("boarCount",eliminate.getBoarCount());
            resultJo.put("sowCount",eliminate.getSowCount());
            resultJo.put("fatpigsCount",eliminate.getFatpigsCount());
        }
        //查询每天记录
        QueryWrapper<EliminateDetail> queryWrapperDetail = new QueryWrapper();
        queryWrapperDetail.eq("farm_code",farmCode);
        queryWrapperDetail.orderByAsc("add_date");
        List<EliminateDetail> EliminateDetailList =  eliminateDetailMapper.selectList(queryWrapperDetail);
        resultJo.put("list",EliminateDetailList);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result save(Map<String, String> paramsMap) throws ParseException {
        //farmCode
        String farmCode = paramsMap.get("farmCode");
        if(farmCode == null){
            farmCode = DEFAULT_FARM_CODE;
        }
        //
        String list = paramsMap.get("list");
        JSONArray listJa = JSONArray.parseArray(list);
        DateUtil dateUtil = new DateUtil();
        //删除原来淘汰明细记录
        QueryWrapper<EliminateDetail> detailQueryWrapper = new QueryWrapper<>();
        detailQueryWrapper.eq("farm_code",farmCode);
        eliminateDetailMapper.delete(detailQueryWrapper);
        //删除原来淘汰总记录
        QueryWrapper<Eliminate> allQueryWrapper = new QueryWrapper<>();
        allQueryWrapper.eq("farm_code",farmCode);
        eliminateMapper.delete(allQueryWrapper);
        //新增记录
        int boarCount=0;
        int sowCount = 0;
        int fatpigsCount = 0;
        for(int a=0;a<listJa.size();a++){
            JSONObject listJo = listJa.getJSONObject(a);
            EliminateDetail eliminateDetail = new EliminateDetail();
            eliminateDetail.setBoarCount(listJo.getInteger("boarCount"));
            eliminateDetail.setSowCount(listJo.getInteger("sowCount"));
            eliminateDetail.setFatpigsCount(listJo.getInteger("fatpigsCount"));
            eliminateDetail.setFarmCode(farmCode);
            eliminateDetail.setAddDate(dateUtil.parseDate(listJo.getString("addDate")));
            eliminateDetailMapper.insert(eliminateDetail);
            boarCount+=listJo.getInteger("boarCount");
            sowCount+=listJo.getInteger("sowCount");
            fatpigsCount+=listJo.getInteger("fatpigsCount");
        }
        //新增总记录
        Eliminate eliminate = new Eliminate();
        eliminate.setBoarCount(boarCount);
        eliminate.setSowCount(sowCount);
        eliminate.setFatpigsCount(fatpigsCount);
        eliminate.setTotalCount(boarCount+sowCount+fatpigsCount);
        eliminate.setFarmCode(farmCode);
        eliminate.setYear(Integer.parseInt(dateUtil.getThisYear()));
        eliminateMapper.insert(eliminate);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        eliminateDetailMapper.deleteById(id);
        return new Result(ResultCode.SUCCESS);
    }
}
