package com.huimv.env.produce.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.BigDecimalCodec;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.produce.entity.FeedDay;
import com.huimv.env.produce.entity.FeedMonth;
import com.huimv.env.produce.mapper.FeedDayMapper;
import com.huimv.env.produce.mapper.FeedMonthMapper;
import com.huimv.env.produce.service.FeedDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.undertow.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-11-03
 */
@Service
public class FeedDayServiceImpl extends ServiceImpl<FeedDayMapper, FeedDay> implements FeedDayService {
    @Autowired
    private FeedDayMapper feedDayMapper;
    @Autowired
    private FeedMonthMapper feedMonthMapper;
    @Autowired
    private static final String globalFarmCode = "330211";


    @Override
    public Result getFeedCount(Map<String, String> paramsMap) throws ParseException {


        //计算今日消耗量、环比
        BigDecimal todayCountBd = new BigDecimal(0);
        BigDecimal todayRatioBd = new BigDecimal(0);
        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 这里写代码计算今日消耗量、环比
        String farmCode = paramsMap.get("farm_code");
        if (farmCode == null) {
            farmCode = globalFarmCode;
        }
        String index = paramsMap.get("index");
        if (index==""||index==null) {
            index = "6";
        }
        QueryWrapper<FeedDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode).orderByDesc("id").last("limit ".concat(index));
        List<FeedDay> feedDays = feedDayMapper.selectList(queryWrapper);
        if (!feedDays.isEmpty()) {
            todayCountBd = feedDays.get(0).getConsumption();
            if (feedDays.size() < 2) {
                todayRatioBd = BigDecimal.ZERO;
            }
            if (feedDays.size()>=2) {
                BigDecimal yesterdayCountBd = feedDays.get(1).getConsumption();
                todayRatioBd = (todayCountBd.subtract(yesterdayCountBd)).divide(yesterdayCountBd,3,BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).setScale(1, BigDecimal.ROUND_HALF_UP);
            }
        } else {
            todayCountBd = BigDecimal.ZERO;
            todayRatioBd = BigDecimal.ZERO;
        }

        //计算本月消耗量、环比
        BigDecimal thisMonthCountBd = new BigDecimal(0);
        BigDecimal thisMonthRatioBd = new BigDecimal(0);
        // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 这里写代码计算本月消耗量、环比
        QueryWrapper<FeedMonth> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_code", farmCode).orderByDesc("id").last("limit ".concat(index));
        List<FeedMonth> feedMonths = feedMonthMapper.selectList(queryWrapper1);
        if (!feedMonths.isEmpty()) {
            thisMonthCountBd = feedMonths.get(0).getConsumption();
            if (feedMonths.size() < 2) {
                thisMonthRatioBd = BigDecimal.ZERO;
            }
            if (feedMonths.size()>=2) {
                BigDecimal LastMonthCountBd = feedMonths.get(1).getConsumption();
                thisMonthRatioBd = (thisMonthCountBd.subtract(LastMonthCountBd)).divide(LastMonthCountBd,3,BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).setScale(1, BigDecimal.ROUND_HALF_UP);
            }
        } else {
            thisMonthCountBd = BigDecimal.ZERO;
            thisMonthRatioBd = BigDecimal.ZERO;
        }

        JSONObject resultJo = new JSONObject();
        //计算历史记录
        List<Object> consumptionList = _getFeedHistoryConsumption(paramsMap);

        //今日消耗量
        resultJo.put("todayCount", todayCountBd.toString());
        //今日环比
        resultJo.put("todayRatio", todayRatioBd.toString());
        //本月消耗量
        resultJo.put("thisMonthCount", thisMonthCountBd.toString());
        //本月环比
        resultJo.put("thisMonthRatio", thisMonthRatioBd.toString());
        //每月消耗记录
        resultJo.put("consumptionList", consumptionList);
        return new Result(ResultCode.SUCCESS, resultJo);
    }

    @Override
    public Result newFeedCount(List<FeedDay> list) {
        QueryWrapper<FeedDay> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        feedDayMapper.delete(queryWrapper);

        for (int i = 0; i <list.size(); i++) {
            FeedDay feedDay = new FeedDay();
            feedDay.setConsumption(list.get(i).getConsumption());
            feedDay.setAddDate(list.get(i).getAddDate());
            feedDay.setFarmCode(globalFarmCode);
            feedDayMapper.insert(feedDay);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result removeFeedCount(Map<String, String> paramSMap) {
        String ids = paramSMap.get("ids");
        String[] idArray = ids.split(",");
        List<String> idList = Arrays.asList(idArray);
        feedDayMapper.deleteBatchIds(idList);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result newDayFeed(Map<String, String> paramsMap) {
        QueryWrapper<FeedDay> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        feedDayMapper.delete(queryWrapper);
        String list = paramsMap.get("list");
        JSONArray listJa = JSONArray.parseArray(list);
        for (int i = 0; i <listJa.size(); i++) {
            FeedDay feedDay = new FeedDay();
            JSONObject listJo = listJa.getJSONObject(i);
            feedDay.setConsumption(listJo.getBigDecimal("consumption"));
            feedDay.setAddDate(listJo.getDate("addDate"));
            feedDay.setFarmCode(globalFarmCode);
            feedDayMapper.insert(feedDay);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listDayFeed(Map<String, String> paramsMap) {
        QueryWrapper<FeedDay> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        queryWrapper.orderByAsc("add_date").orderByAsc("id");
        List<FeedDay> feedMonthList = feedDayMapper.selectList(queryWrapper);
        return new Result(ResultCode.SUCCESS,feedMonthList);
    }


    private List<Object> _getFeedHistoryConsumption(Map<String, String> paramsMap) throws ParseException {
        String months = paramsMap.get("months");
        String days = paramsMap.get("days");
        String farmCode = paramsMap.get("farm_code");
        if (farmCode == null) {
            farmCode = globalFarmCode;
        }
        List<Object> consumptionList = new ArrayList<>();
        DateUtil dateUtil = new DateUtil();
        if (months == null && days != null) {
            //读取N天历史记录，以正序返回
            QueryWrapper<FeedDay> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_code", farmCode).orderByAsc("id").last("limit ".concat(days));
            List<FeedDay> feedDaysList = feedDayMapper.selectList(queryWrapper);
            JSONArray dayListJa = new JSONArray();
            for(int a=0;a<feedDaysList.size();a++){
                FeedDay feedDay = feedDaysList.get(a);
                JSONObject newJo = new JSONObject();
                dayListJa.add(newJo);
                newJo.put("datetime",dateUtil.formatDateText(feedDay.getAddDate()));
                newJo.put("water",feedDay.getConsumption());
            }
            consumptionList.addAll(dayListJa);
        } else if (months != null && days == null) {
            //读取N月历史记录，以正序返回
            QueryWrapper<FeedMonth> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("farm_code", farmCode).orderByAsc("id").last("limit ".concat(months));
            List<FeedMonth> feedMonthsList = feedMonthMapper.selectList(queryWrapper2);
            JSONArray monthListJa = new JSONArray();
            for(int a=0;a<feedMonthsList.size();a++){
                FeedMonth feedMonth = feedMonthsList.get(a);
                JSONObject newJo = new JSONObject();
                monthListJa.add(newJo);
                newJo.put("datetime",feedMonth.getYear()+"-"+feedMonth.getMonth());
                newJo.put("water",feedMonth.getConsumption());
            }
            consumptionList.addAll(monthListJa);
        }
        return consumptionList;
    }

}
