package com.huimv.env.produce.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.produce.entity.FeedDay;
import com.huimv.env.produce.entity.FeedMonth;
import com.huimv.env.produce.mapper.FeedMonthMapper;
import com.huimv.env.produce.service.FeedMonthService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-11-03
 */
@Service
public class FeedMonthServiceImpl extends ServiceImpl<FeedMonthMapper, FeedMonth> implements FeedMonthService {

    @Autowired
    private FeedMonthMapper feedMonthMapper;
    @Autowired
    private static final String globalFarmCode = "330211";

    @Override
    public Result removeMonthCount(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] idArray = ids.split(",");
        List<String> idList = Arrays.asList(idArray);
        feedMonthMapper.deleteBatchIds(idList);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result newMonthFeed(Map<String, String> paramsMap) {
        QueryWrapper<FeedMonth> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        feedMonthMapper.delete(queryWrapper);
        String list = paramsMap.get("list");
        JSONArray listJa = JSONArray.parseArray(list);
        for (int i = 0; i <listJa.size(); i++) {
            FeedMonth feedMonth = new FeedMonth();
            JSONObject listJo = listJa.getJSONObject(i);
            feedMonth.setConsumption(listJo.getBigDecimal("consumption"));
            String addDate = listJo.getString("addDate");
            String[]  addDateArray = addDate.split("-");
            feedMonth.setYear(Integer.parseInt(addDateArray[0]));
            feedMonth.setMonth(Integer.parseInt(addDateArray[1]));
            feedMonth.setFarmCode(globalFarmCode);
            feedMonthMapper.insert(feedMonth);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listMonthFeed(Map<String, String> paramsMap) {
        QueryWrapper<FeedMonth> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        queryWrapper.orderByAsc("year").orderByAsc("month");
        List<FeedMonth> feedMonthList = feedMonthMapper.selectList(queryWrapper);
        JSONArray monthJa = new JSONArray();
        for(int a=0;a<feedMonthList.size();a++){
            FeedMonth feedMonth = feedMonthList.get(a);
            JSONObject newJo = new JSONObject();
            monthJa.add(newJo);
            newJo.put("consumption",feedMonth.getConsumption());
            int month = feedMonth.getMonth();
            String monthStr = ""+month;
            if(month<10){
                monthStr ="0"+monthStr;
            }
            newJo.put("addDate",feedMonth.getYear()+"-"+monthStr);
        }
        return new Result(ResultCode.SUCCESS,monthJa);
    }

    @Override
    public Result newFeedMonthCount(List<FeedMonth> list) {
        QueryWrapper<FeedMonth> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code", globalFarmCode);
        feedMonthMapper.delete(queryWrapper);

        for (int i = 0; i <list.size(); i++) {
            FeedMonth feedMonth = new FeedMonth();
            feedMonth.setConsumption(list.get(i).getConsumption());
            feedMonth.setMonth(list.get(i).getMonth());
            feedMonth.setYear(list.get(i).getYear());
            feedMonth.setFarmCode(globalFarmCode);
            feedMonthMapper.insert(feedMonth);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
