package com.huimv.env.produce.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.produce.entity.Reminder;
import com.huimv.env.produce.entity.Sales;
import com.huimv.env.produce.entity.SalesDetail;
import com.huimv.env.produce.mapper.ReminderMapper;
import com.huimv.env.produce.service.ReminderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-11-02
 */
@Service
public class ReminderServiceImpl extends ServiceImpl<ReminderMapper, Reminder> implements ReminderService {
    //默认牧场
    private static final String DEFAULT_FARM_CODE = "330211";
    @Autowired
    private ReminderMapper reminderMapper;

    @Override
    public Result getReminder(Map<String, String> paramsMap) {
        //
        String farmCode = paramsMap.get("farmCode");
        if(farmCode == null){
            farmCode = DEFAULT_FARM_CODE;
        }
        //
        QueryWrapper<Reminder> totalQueryWrapper = new QueryWrapper();
        totalQueryWrapper.eq("farm_code",farmCode);
        totalQueryWrapper.orderByDesc("id");
        List<Reminder> reminderList = reminderMapper.selectList(totalQueryWrapper);
        return new Result(ResultCode.SUCCESS,reminderList);
    }

    @Override
    public Result save(Map<String, String> paramsMap) {
//
        String farmCode = paramsMap.get("farmCode");
        if(farmCode == null){
            farmCode = DEFAULT_FARM_CODE;
        }
        //删除原先明细表
        QueryWrapper<Reminder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_code",farmCode);
        reminderMapper.delete(queryWrapper);
        //新增明细表
        String list = paramsMap.get("list");
        JSONArray listJa = JSONArray.parseArray(list);
        int count = 0;
        for(int a=listJa.size()-1;a>=0;a--){
            JSONObject listJo = listJa.getJSONObject(a);
            Reminder reminder = new Reminder();
            reminder.setContent(listJo.getString("content"));
            reminder.setFarmCode(farmCode);
            reminderMapper.insert(reminder);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listTodayPlan(Map<String, String> paramsMap) {
//        String farmCode = paramsMap.get("farmCode");
//        if(farmCode == null){
//            farmCode = DEFAULT_FARM_CODE;
//        }
        String farmCode = DEFAULT_FARM_CODE;
        //
        QueryWrapper<Reminder> totalQueryWrapper = new QueryWrapper();
        totalQueryWrapper.eq("farm_code",farmCode);
        totalQueryWrapper.orderByDesc("id");
        List<Reminder> reminderList = reminderMapper.selectList(totalQueryWrapper);
        JSONArray dataJa = new JSONArray();
        for(int a=0;a<reminderList.size();a++){
            Reminder reminder = reminderList.get(a);
            JSONObject reminderJo = new JSONObject();
            reminderJo.put("data",reminder.getContent());
            dataJa.add(reminderJo);
        }

        JSONArray emptyJa = new JSONArray();
        JSONObject resultJo = new JSONObject();
        resultJo.put("records",dataJa);
        resultJo.put("total",dataJa.size());
        resultJo.put("size",10);
        resultJo.put("current",1);
        resultJo.put("orders",emptyJa);
        resultJo.put("optimizeCountSql",true);
        resultJo.put("searchCount",true);
        resultJo.put("countId",null);
        resultJo.put("maxLimit",null);
        resultJo.put("pages",1);
        return new Result(ResultCode.SUCCESS,resultJo);
    }
}
